{% extends "users/layout.html" %}
{% block users_title %}{{ _("Reset password") }}{% endblock %}
{% block users_content %}
<{ form_tag form }>
    <{ form_field form.email }/>
    <{ form_btn _("Send email to reset password") }/>
</{}>
{% endblock %}
